import os
from pathlib import Path
from dotenv import load_dotenv
load_dotenv()

SAVES = load_saves()

HOST=os.getenv("HOST")
REPO=os.getenv("REPO")
USERNAME=os.getenv("SSH_USERNAME")
PASSWORD=os.getenv("SSH_PASSWORD")

from fabric import Connection

def get_id_rsa():
    home_dir = Path(os.path.expanduser('~'))
    id_rsa_path = home_dir / ".ssh" / "id_rsa.pub"
    with open(id_rsa_path) as f:
        content = f.read()
    return content

def run_user_setup():
    super_user = input("Type in ssh user: ")
    password = input(f"Type in {super_user}@{HOST} ssh password:")
    
    ssh_pub_file = get_id_rsa()
    command = f"""
    #Create user
    useradd -m -G sudo,www-data {USERNAME} &&\\ \n\
    echo -e \"{PASSWORD}\\n{PASSWORD}\\n\" | passwd {USERNAME} \n\
    chsh -s /bin/bash {USERNAME} \n\
    #Create SSH directory \n\
    mkdir /home/{USERNAME}/.ssh \n\
    cd /home/{USERNAME}/.ssh \n\
    #Copy over public key \n\
    touch authorized_keys \n\
    echo \"{ssh_pub_file}\" > authorized_keys \n\
    #Set SSH key permissions \n\
    chmod 700 /home/{USERNAME}/.ssh \n\
    #Change file/folder ownership to new user \n\
    chown -R {USERNAME}:{USERNAME} /home/{USERNAME} \n\
    exit
    """

    result = Connection(f'{super_user}@{HOST}').run(command, hide=True)
    msg = "Ran {0.command!r} on {0.connection.host}, got stdout:\n{0.stdout}"
    return msg.format(result)

def add_key():
    ssh_pub_file = get_id_rsa()
    command = f"""
    echo \"{ssh_pub_file}\" > ~/.ssh/authorized_keys \n\
    exit
    """
    print(command)
    result = Connection(f'{USERNAME}@{HOST}', connect_kwargs={"password" : PASSWORD}).run(command, hide=False)
    msg = "Ran {0.command!r} on {0.connection.host}, got stdout:\n{0.stdout}"
    return msg.format(result)


if __name__=="__main__":
    run_user_setup()


import os
import json
from pathlib import Path 


def get_notes_path():
    return relative_to_absolute(load_saves()["location"])

def relative_to_absolute(relative):
    home_dir = Path(os.path.expanduser('~'))
    notes_path = str(Path(f"{str(home_dir)}/{relative}"))
    return notes_path

def load_saves():
    try:
        with open("data.json") as f:
            SAVES = json.load(f)
    except:
        SAVES = dict()
    return SAVES


def get_all_files(path, ignore=[]):
    output = []
    for root, directories, files in os.walk(path, topdown=False):
        for name in files:
            path = os.path.join(root, name)
            if not any([i in path for i in ignore]):
                output.append(path)
    return output

def file_to_words(path):
    with open(path) as f:
        content = f.read()
    return content.split()
    






import os
from spekter.monitor import DirLoop
from pathlib import Path
from spekter.git import push as git_push, pull as git_pull, commit, get_repo, commit_and_push
from spekter.utils import get_all_files, relative_to_absolute, get_notes_path, load_saves
from spekter.monitor import dump_saves
import sys
import datetime
import click
import json

@click.group()
def cli():
    pass

@click.command()
def init():
    host = input("Type in server IP: ")
    repo = input("Type in the repository name (main.git): ")
    location = input("Type in notes folder location relative to home (Documents/Notes): ")
    SAVES = load_saves()
    SAVES["host"] = host
    SAVES["repo"] = repo
    SAVES["location"] = location
    SAVES["last_checked"] = datetime.datetime.now().timestamp()
    SAVES["files"] = []
    with open("data.json", "w") as f:
        json.dump(SAVES, f)

@click.command()
def push():
    commit_and_push()
    click.echo('Pushing spekter changes')

@click.command()
def pull():
    notes_path = get_notes_path()
    repo = get_repo(notes_path)
    git_pull(repo)
    click.echo('Pulling spekter changes')

@click.command()
def listen():
    notes_path = get_notes_path()

    def notify(x):
        print(x)

    l = DirLoop(path=notes_path)
    repo = get_repo(notes_path)
    l.set_save_func(commit_and_push)
    l.set_open_func(notify, "OPEN")
    l.start()

cli.add_command(init)
cli.add_command(push)
cli.add_command(pull)
cli.add_command(listen)



import os
from pathlib import Path
import datetime
import json
import time

from spekter.utils import get_all_files, load_saves

def get_last_modified(path):
    ts = os.path.getmtime(path)
    dt = datetime.datetime.fromtimestamp(ts)
    return dt

def get_last_accessed(path):
    ts = os.path.getatime(path)
    dt = datetime.datetime.fromtimestamp(ts)
    return dt

def saved_files(files, SAVES):
    output = []
    for file in files:
        if file not in SAVES["files"]:
            output.append(file)
        else:
            if get_last_modified(file).timestamp() > SAVES["last_checked"]:
                output.append(file)
    return output

def opened_files(files, SAVES):
    output = []
    for file in files:
        if get_last_accessed(file).timestamp() > SAVES["last_checked"]:
            output.append(file)
    return output   

def dump_saves(files):
    SAVES = load_saves()
    SAVES["last_checked"] = datetime.datetime.now().timestamp()
    SAVES["files"] = files
    with open("data.json", "w") as f:
        json.dump(SAVES, f)

class DirLoop:

    def __init__(self, path, wait=5):
        self.path = path
        self.wait = wait

    def set_save_func(self, function, *args, **kwargs):
        self.save_func = function
        self.save_args = args
        self.save_kwargs = kwargs
    
    def set_open_func(self, function, *args, **kwargs):
        self.open_func = function
        self.open_args = args
        self.open_kwargs = kwargs

    def trigger_save(self):
        self.save_func(*self.save_args, **self.save_kwargs)

    def trigger_open(self):
        self.open_func(*self.open_args, **self.open_kwargs)

    def start(self):
        SAVES = load_saves()
        while True: 
            try:
                files = get_all_files(self.path, ignore=[".DS_Store", "/data", "/.git", "sonic.cfg", ".gitignore", "\\data", "\\.git"])
                saved = saved_files(files, SAVES)
                opened = opened_files(files, SAVES)
                opened = [i for i in opened if i not in saved]
                #SAVED
                if len(saved) > 0:
                    if self.save_func:
                        print("Triggering save function.")
                        self.trigger_save()
                #OPENED
                if len(opened) > 0:
                    if self.open_func:
                        print("Triggering open function.")
                        self.trigger_open()
                if len(opened) > 0 or len(saved) > 0:
                    dump_saves(files)
                    SAVES = load_saves()
                    print(SAVES)
                time.sleep(self.wait)
            except KeyboardInterrupt:
                break    

if __name__=="__main__":
    def notify(x):
        print(x)

    home_dir = Path(os.path.expanduser('~'))
    dir_path = f"{str(home_dir)}/Documents/Notes"

    l = DirLoop(path=dir_path, wait=5)
    l.set_save_func(notify, x="SAVED")
    l.set_open_func(notify, x="OPENED")
    l.start()



import os
from dulwich import porcelain
from dulwich.repo import Repo
from pathlib import Path
from spekter.utils import get_all_files, load_saves

from dotenv import load_dotenv
load_dotenv()

SAVES = load_saves()

HOST=SAVES.get("host")
REPO=SAVES.get("repo")
USERNAME="spekter"

def create_new_repo(path): 
    repo = porcelain.init(path)
    return repo

def clone_repo(url, path):
    porcelain.clone(url, path)

def get_repo(path):
    repo = Repo(path)
    return repo

def commit(repo, files):
    for file in files:
        porcelain.add(repo, files)
        porcelain.commit(repo, b"Automated commit")

def commit_and_push():
    notes_path = get_notes_path()
    files = get_all_files(notes_path, ignore=[".DS_Store", "/data", "/.git", "sonic.cfg", "\\.git", "\\data"])
    repo = get_repo(notes_path)
    commit(repo, files)
    git_push(repo)

def push(repo):
    r = porcelain.push(repo.path, f"{USERNAME}@{HOST}:{REPO}", "master")

def pull(repo):
    r = porcelain.pull(repo.path, f"{USERNAME}@{HOST}:{REPO}")

if __name__=="__main__":
    home_dir = Path(os.path.expanduser('~'))
    repo = get_repo(f"{str(home_dir)}/Documents/Notes")
    relative_files = ["." + file.replace(repo.path, "") for file in files]
    commit(repo, files)
    push(repo)




import os
from pathlib import Path

from sonic import SearchClient
from sonic import IngestClient
from spekter.utils import file_to_words, get_all_files

def clean_file(file):
    return file.replace(" ", "<space>")

def injest(files):
    with IngestClient("127.0.0.1", 1491, "SecretPassword") as ingestcl:
        ingestcl.flush("files", "spekter")
        for file in files:
            words = file_to_words(file)
            for idx, word in enumerate(words):
                try:
                    ingestcl.push("files", "spekter", f'{clean_file(file)}:{str(idx)}', word)
                except Exception as e:
                    print(f"Failed to push: {file}:{str(idx)} -> {word}")
                    print(e)

def search(query):
    with SearchClient("127.0.0.1", 1491, "SecretPassword") as querycl:
        return querycl.query("files", "spekter", query)

def parse_search(results, before=7, after=7):
    output = []
    for result in results:
        file_path, word_idx = result.split(":")
        file_path = file_path.replace("<space>", " ")
        words = file_to_words(file_path)
        a = max(int(word_idx) - before, 0)
        b = min(int(word_idx) + after, len(words)-1)
        output.append({
            "file_path" : file_path, 
            "word" : words[int(word_idx)], 
            "phrase" : " ".join(words[a:b])
        })
    return output

if __name__=="__main__":
    home_dir = Path(os.path.expanduser('~'))
    files = get_all_files(f"{str(home_dir)}/Documents/Notes", ignore=[".DS_Store", "/data", "/.git", "sonic.cfg"])
    injest(files)
    print(parse_search(search("Template")))



from setuptools import setup, find_packages

setup(
    name='spekter',
    version='0.1.1',
    packages=find_packages(), 
    include_package_data=True, 
    install_requires=[
        'Click',
        'Flask==1.1.2',
        'sonic_client==0.0.5',
        'dulwich==0.20.21',
        'Fabric==2.6.0',
        'python-dotenv==0.17.1'
    ],
    entry_points={
        'console_scripts': [
            'spekter = spekter.main:cli',
        ],
    },
)


from flask import Flask, jsonify, request
from spekter.sonic_utils import search, parse_search

app = Flask(__name__)

@app.route("/search", methods=["POST"])
def sonic_search():
    query = request.form.get("query")
    return jsonify(
        parse_search(search(query))
    )

if __name__=="__main__":
    app.run(host="127.0.0.1", port="1492", debug=True)